/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * Test image opening/closing
 * @test ImageOpenTest
 * @summary Unit test for JVM_ImageOpen() method
 * @library /testlibrary /../../test/lib
 * @build ImageOpenTest
 * @run main ClassFileInstaller sun.hotspot.WhiteBox
 *                              sun.hotspot.WhiteBox$WhiteBoxPermission
 * @run main/othervm -Xbootclasspath/a:. -XX:+UnlockDiagnosticVMOptions -XX:+WhiteBoxAPI ImageOpenTest
 */

import java.io.File;
import java.nio.ByteOrder;
import sun.hotspot.WhiteBox;
import static jdk.test.lib.Asserts.*;

public class ImageOpenTest {

    public static final WhiteBox wb = WhiteBox.getWhiteBox();

    public static void main(String... args) throws Exception {
        String javaHome = System.getProperty("java.home");
        String nonexistentImageFile = javaHome + "/lib/modules/nonexistent.jimage";
        String bootmodulesImageFile = javaHome + File.separator + "lib" + File.separator
                + "modules" + File.separator + "bootmodules.jimage";

        if (!(new File(bootmodulesImageFile)).exists()) {
            System.out.printf("Test skipped.");
            return;
        }

        boolean bigEndian = ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN;

        // open nonexistent image
        long id = wb.imageOpenImage(nonexistentImageFile, bigEndian);
        assertTrue(id == 0L, "Failed. Get id " + id + "instead of 0 on opening nonexistent file\n");
        wb.imageCloseImage(id);

        // open bootmodules image
        id = wb.imageOpenImage(bootmodulesImageFile, bigEndian);
        assertFalse(id == 0, "Failed. Get id 0 on opening bootmodules.jimage");
        wb.imageCloseImage(id);

        // non-native endian
        id = wb.imageOpenImage(bootmodulesImageFile, !bigEndian);
        assertFalse(id == 0, "Failed. Get id 0 on opening bootmodules.jimage with non-native endian");
        wb.imageCloseImage(id);

        //
        // open several times
        //
        id = wb.imageOpenImage(bootmodulesImageFile, bigEndian);
        long id1 = wb.imageOpenImage(bootmodulesImageFile, bigEndian);
        long id2 = wb.imageOpenImage(bootmodulesImageFile, bigEndian);
        assertTrue((id == id1) && (id == id2), "Failed. Open thee times with ids " + id + " " + id1 + " " + id1);

        wb.imageCloseImage(id);
        wb.imageCloseImage(id1);
        wb.imageCloseImage(id2);
    }
}
